/*
 * MIT License
 *
 * Copyright (c) 2019 Alexey Edelev <semlanik@gmail.com>
 *
 * This file is part of NeuralNetwork project https://git.semlanik.org/semlanik/NeuralNetwork
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this
 * software and associated documentation files (the "Software"), to deal in the Software
 * without restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of the Software, and
 * to permit persons to whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "dense.h"
#include <QtGlobal>

//0 -  3  Version = 1          (uint32)
//4       'G'                  (byte)
//5       'F'                  (byte)
//6       'A'                  (byte)
//7       0                    (byte)
//8 - 15  number of rows       (int64)
//16 - 23  number of columns    (int64)
//24 - 31  0                    (int64)
//32 - 39  0                    (int64)
//40 - ..  matrix data elements (float64)
//        [0,0] [0,1] ... [0,ncols-1]
//        [1,0] [1,1] ... [1,ncols-1]
//        ...
//        [nrows-1,0] ... [nrows-1,ncols-1]

Dense::Dense(const QByteArray &data) : AbstractDense(*(int64_t *)(data.data() + 8), *(int64_t *)(data.data() + 16), data)
{
}

double Dense::rawValue(int i) const
{
    return *(double *)(m_data.data() + 40 + i * sizeof(double));
}


template<>
template<>
double AbstractDense<QByteArray>::value<double>(int row, int column) const
{
    const char *dataPtr = m_data.data() + 40 + ((m_columns - 1) * row + column + row) * sizeof(double);
    return *(double *)dataPtr;
}
